###############################################
# Code to replicate results provided in the   #
# supplementary appendix for:                 #
# "A Modern Peace? Schumpeter, the Decline of #    
# Conﬂict, and the Investment-War Tradeoff"   #
#                                             #
# J. Tyson Chatagnier and Emanuele Castelli   #
#                                             #
# Forthcoming in Political Research Quarterly #
#                                             #
# File created 8 August 2016                  #
###############################################

rm(list = ls())

library(foreign)
library(ggplot2)
library(apsrtable)
library(reshape)
library(Amelia)

 setwd("C:/OneDrive/Stat Programs/R/R Code/Industrial Peace/Replication Files/")
# setwd("path")
                        # Use to change default directory

source("modern_peace_auxiliary_functions.r")

################
# Read in Data #
################

monadic <- read.dta("monadic_data_set.dta")

dyadic <- read.dta("dyadic_data_set.dta")

####################################
# Table A1. Descriptive Statistics #
####################################

vars.mon <- data.frame(Industrialization      = monadic$industry,
                       Service                = monadic$service,
                       Polity                 = monadic$polity2,
                       Public_Property        = monadic$public,
                       Capital_Openness       = monadic$open,
                       Contract_Intensiveness = monadic$cie
                       )

vars.dy <- data.frame(Industrialization_H      = dyadic$ind_hi,
                      Service_Ind_H            = dyadic$serv_ind_hi,
                      Polity_L                 = dyadic$lodem,
                      Public_Property_H        = dyadic$publich,
                      Capital_Openness_L       = dyadic$openl,
                      Contract_Intensiveness_L = dyadic$cielo
                      )

GetDesc(vars.mon)

GetDesc(vars.dy)

################################
# Table A2. Correlation Matrix #
################################

cor.df <- data.frame(Industrialization = monadic$industry,
                     Service           = monadic$service,
                     Democracy         = monadic$polity2,
                     GDP               = log(monadic$gdp),
                     Property          = monadic$public,
                     Openness          = monadic$open,
                     Contract          = monadic$cie
                     )

cor(cor.df,
    use = "complete.obs"
    )

#########################################
# Fig A1. Monadic effect of peace years #
#########################################

t.seq <- seq(from = 0,
             to   = 200,
             by   = 1
             )

load("monadic_mods.rdata")
                        # Load results saved during initial monadic analysis

haz.base.mon <- GetHaz(t.seq, 
                       monad.base$coef, 
                       vcov.base.mon, 
                       c(10, 
                         11, 
                         12
                         )
                       )

haz.mcd.mon <- GetHaz(t.seq, 
                      monad.mcd$coef, 
                      vcov.mcd.mon, 
                      c(11, 
                        12, 
                        13
                        )
                      )

haz.open.mon <- GetHaz(t.seq, 
                       monad.open$coef, 
                       vcov.open.mon, 
                       c(11, 
                         12, 
                         13
                         )
                       )

haz.cont.mon <- GetHaz(t.seq, 
                       monad.cont$coef, 
                       vcov.cont.mon, 
                       c(11, 
                         12, 
                         13
                         )
                       )


haz.mon <- data.frame(Years = t.seq,
                      val   = c(haz.base.mon$haz,
                                haz.mcd.mon$haz,
                                haz.open.mon$haz,
                                haz.cont.mon$haz
                                ),
                      lo    = c(haz.base.mon$lo,
                                haz.mcd.mon$lo,
                                haz.open.mon$lo,
                                haz.cont.mon$lo
                                ),
                      hi    = c(haz.base.mon$hi,
                                haz.mcd.mon$hi,
                                haz.open.mon$hi,
                                haz.cont.mon$hi
                                ),
                      Model = rep(c("Baseline",
                                    "Public\nProperty",
                                    "Openness",
                                    "Contract\nIntensiveness"
                                    ),
                                    each = length(t.seq)
                                  )
                      )

haz.mon$Model <- factor(haz.mon$Model, 
                        c("Baseline",
                          "Public\nProperty",
                          "Openness",
                          "Contract\nIntensiveness"
                          )
                        )

pdf("hazard_effects_monadic.pdf",
    height = 7,
    width  = 8
    )
  ggplot(data = haz.mon,
         aes(x      = Years,
             y      = val,
             colour = Model
             )
         ) +
    geom_line(size = 1.25) +
    geom_ribbon(data = haz.mon,
                aes(ymin = lo,
                    ymax = hi,
                    fill = Model
                    ),
                alpha  = 0.3,
                colour = NA
                ) +
    ylab(expression(paste("Effect on X", 
                          beta
                          )
                    )
         ) +
    xlab("Years since conflict") +
    ggtitle("Marginal Effect of Time on Linear Predictor") +
    theme_bw()
dev.off()

########################################
# Fig A2. Dyadic effect of peace years #
########################################

load("dyadic_mods.rdata")
                        # Load results saved during initial dyadic analysis

haz.base.dy <- GetHaz(t.seq, 
                      dyad.base$coef, 
                      vcov.base.dy, 
                      c(13, 
                        14, 
                        15
                        )
                      )

haz.mcd.dy <- GetHaz(t.seq, 
                     dyad.mcd$coef, 
                     vcov.mcd.dy, 
                     c(14, 
                       15, 
                       16
                       )
                     )

haz.open.dy <- GetHaz(t.seq, 
                      dyad.open$coef, 
                      vcov.open.dy, 
                      c(14, 
                        15, 
                        16
                        )
                      )

haz.cont.dy <- GetHaz(t.seq, 
                      dyad.cont$coef, 
                      vcov.cont.dy, 
                      c(14, 
                        15, 
                        16
                        )
                      )


haz.dy <- data.frame(Years = t.seq,
                     val   = c(haz.base.dy$haz,
                               haz.mcd.dy$haz,
                               haz.open.dy$haz,
                               haz.cont.dy$haz
                               ),
                     lo    = c(haz.base.dy$lo,
                               haz.mcd.dy$lo,
                               haz.open.dy$lo,
                               haz.cont.dy$lo
                               ),
                     hi    = c(haz.base.dy$hi,
                               haz.mcd.dy$hi,
                               haz.open.dy$hi,
                               haz.cont.dy$hi
                               ),
                     Model = rep(c("Baseline",
                                   "Public\nProperty",
                                   "Openness",
                                   "Contract\nIntensiveness"
                                   ),
                                   each = length(t.seq)
                                 )
                     )

haz.dy$Model <- factor(haz.dy$Model, 
                       c("Baseline",
                         "Public\nProperty",
                         "Openness",
                         "Contract\nIntensiveness"
                         )
                       )

pdf("hazard_effects_dyadic.pdf",
    height = 7,
    width  = 8
    )
  ggplot(data = haz.dy,
         aes(x      = Years,
             y      = val,
             colour = Model
             )
         ) +
    geom_line(size = 1.25) +
    geom_ribbon(data = haz.dy,
                aes(ymin = lo,
                    ymax = hi,
                    fill = Model
                    ),
                alpha  = 0.3,
                colour = NA
                ) +
    ylab(expression(paste("Effect on X", 
                          beta
                          )
                    )
         ) +
    xlab("Years since conflict") +
    ggtitle("Marginal Effect of Time on Linear Predictor") +
    theme_bw()
dev.off()

###########################
# Impute monadic data set #
###########################

monadic.full <- read.dta("monadic_data_full.dta")
                        # Get full monadic data set, running from year 1 to 2015

monadic.full <- monadic.full[monadic.full$year >= 1930 & monadic.full$year <= 2010, ]
                        # Trim the observations so that we begin in 1930, 
                        # and end in 2010 (the last year for which the 
                        # dependent variable is available)

set.seed(2016)

monadic.mi <- ImputeMonadic(monadic.full)

###################################
# Table A3. Monadic Interpolation #
###################################

monadic.interp <- MonadicMI(data.imp = monadic.mi,
                            type     = "interpolate"
                            )

MakeTable(monadic.interp)

###################################
# Table A4. Monadic Extrapolation #
###################################

monadic.extrap <- MonadicMI(data.imp = monadic.mi,
                            type     = "extrapolate"
                            )

MakeTable(monadic.extrap)

###########################
# Impute dyadic data set #
###########################

dyadic.full <- read.dta("dyadic_data_full.dta")
                        # Get full dyadic data set, running from year 1870 to 2009

dyadic.full <- dyadic.full[dyadic.full$year >= 1930, ]
                        # Trim these data so that they begin in 1930 as well

set.seed(2016)
                        # Imputation process can be computationally demanding,
                        # possibly requiring separate imputations of the 
                        # two data sets. So we set seeds for both just in case.

dyadic.mi <- ImputeDyadic(dyadic.full)

##################################
# Table A5. Dyadic Interpolation #
##################################

dyadic.interp <- DyadicMI(data.imp = dyadic.mi,
                          type     = "interpolate"
                          )

MakeTable(dyadic.interp)

###################################
# Table A6. Dyadic Extrapolation #
###################################

dyadic.extrap <- DyadicMI(data.imp = dyadic.mi,
                          type = "extrapolate"
                          )

MakeTable(dyadic.extrap)
