###############################################
# Code to replicate dyadic analysis for:      #
# "A Modern Peace? Schumpeter, the Decline of #    
# Conﬂict, and the Investment-War Tradeoff"   #
#                                             #
# J. Tyson Chatagnier and Emanuele Castelli   #
#                                             #
# Forthcoming in Political Research Quarterly #
#                                             #
# File created 29 July 2016                   #
###############################################

rm(list=ls())

library(foreign)
library(ggplot2)
library(apsrtable)

# setwd("path")
                        # Use to change default directory

source("modern_peace_auxiliary_functions.r")

########
# Data #
########

dyadic <- read.dta("dyadic_data_set.dta")

#####################
# Prepare variables #
#####################

dyadic <- dyadic[dyadic$fatal_ongo == 0, ]
                        # Drop dyad-years with ongoing conflicts

mod.df <- data.frame(ind_hi       = dyadic$ind_hi,
                     serv_ind_hi  = dyadic$serv_ind_hi,
                     logdp        = dyadic$gdp_lo,
                     Mrat         = dyadic$Mrat,
                     majpow       = dyadic$majpow,
                     lodem        = dyadic$lodem,
                     hidem        = dyadic$hidem,
                     cont         = dyadic$contig,
                     nuclear      = dyadic$nuclear,
                     allies       = dyadic$alliance,
                     affinity     = dyadic$s2uni,
                     dyadic_trade = dyadic$lntrade,
                     t_fatal      = dyadic$t_fatal,
                     t2_fatal     = dyadic$t_fatal ^ 2,
                     t3_fatal     = dyadic$t_fatal ^ 3,
                     public_hi    = dyadic$publich,
                     capopenl     = dyadic$openl,
                     cie_lo       = dyadic$cielo,
                     mid          = dyadic$fatal_onset,
                     dyadid       = dyadic$dyadid,
                     year         = dyadic$year
                     )

mod.df$dyadic_trade <- ifelse(is.na(mod.df$dyadic_trade),
                              0,
                              mod.df$dyadic_trade
                              )

###################
# Estimate models #
###################

mod.base <- glm(mid ~ ind_hi + serv_ind_hi + logdp + Mrat + 
                      majpow + lodem + hidem + cont + allies + 
                      dyadic_trade + affinity + t_fatal + 
                      t2_fatal + t3_fatal,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

mod.mcd <- glm(mid ~ ind_hi + serv_ind_hi + logdp + Mrat + 
                     majpow + lodem + hidem + cont + allies + 
                     dyadic_trade + affinity + public_hi + 
                     t_fatal + t2_fatal + t3_fatal,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

mod.open <- glm(mid ~ ind_hi + serv_ind_hi + logdp + Mrat + 
                      majpow + lodem + hidem + cont + allies + 
                      dyadic_trade + affinity + capopenl + 
                      t_fatal + t2_fatal + t3_fatal,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

mod.cont <- glm(mid ~ ind_hi + serv_ind_hi + logdp + Mrat + 
                      majpow + lodem + hidem + cont + allies + 
                      dyadic_trade + affinity + cie_lo + 
                      t_fatal + t2_fatal + t3_fatal,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

vcov.base <- ClusterVarCov(mod.base,
                           .data   = mod.df,
                           cluster = as.character(mod.df$dyadid)
                           )

vcov.mcd <- ClusterVarCov(mod.mcd,
                          .data   = mod.df,
                          cluster = as.character(mod.df$dyadid)
                          )

vcov.open <- ClusterVarCov(mod.open,
                           .data   = mod.df,
                           cluster = as.character(mod.df$dyadid)
                           )

vcov.cont <- ClusterVarCov(mod.cont,
                           .data   = mod.df,
                           cluster = as.character(mod.df$dyadid)
                           )

###########
# Table 1 #
###########

mod.base$se <- sqrt(diag(vcov.base))
mod.mcd$se <- sqrt(diag(vcov.mcd))
mod.open$se <- sqrt(diag(vcov.open))
mod.cont$se <- sqrt(diag(vcov.cont))


apsrtable(mod.base,
          mod.mcd,
          mod.open,
          mod.cont,
          stars = "default",
          se = "robust"
          )
                        # Note: formatting is not identical to the
                        # table in the article, but the substance
                        # is the same.

#####################################
# Save models to generate plots for #
# appendix later.                   #
#####################################

dyad.base <- mod.base
dyad.mcd <- mod.mcd
dyad.open <- mod.open
dyad.cont <- mod.cont

vcov.base.dy <- vcov.base
vcov.mcd.dy <- vcov.mcd
vcov.open.dy <- vcov.open
vcov.cont.dy <- vcov.cont
                        # Rename to distinguish from monadic models

save(dyad.base,
     dyad.mcd,
     dyad.open,
     dyad.cont,
     vcov.base.dy,
     vcov.mcd.dy,
     vcov.open.dy,
     vcov.cont.dy,
     file = "dyadic_mods.rdata"
     )
                        # Save these data frames

###########################
# Get substantive effects #
###########################

range.ind <- MinMax(mod.df$ind_hi,
                    n.out = 1000
                    )

range.public <- MinMax(mod.df$public_hi,
                       n.out = 1000
                       )

range.open <- MinMax(mod.df$capopenl,
                     n.out = 1000
                     )

range.cie <- MinMax(mod.df$cie_lo,
                    n.out = 1000
                    )

#########################################
# Average effects for industrialization #
# and democracy in the baseline model   #
#########################################

ind.base <- AvgPreds(mod        = mod.base,
                     varcov     = vcov.base,
                     var.change = 2,
                     pol.rel    = TRUE,
                     change.vec = range.ind
                     )

dem.base <- AvgPreds(mod        = mod.base,
                     varcov     = vcov.base,
                     var.change = 7,
                     pol.rel    = TRUE,
                     change.lo  = -10,
                     change.hi  = 10
                     )
 
##########################################
# Average effects for industrialization, #
# capitalism, and democracy in the       #
# McDonald model                         #
##########################################


ind.mcd <- AvgPreds(mod        = mod.mcd,
                    varcov     = vcov.mcd,
                    var.change = 2,
                    pol.rel    = TRUE,
                    change.vec = range.ind
                    )

dem.mcd <- AvgPreds(mod        = mod.mcd,
                    varcov     = vcov.mcd,
                    var.change = 7,
                    pol.rel    = TRUE,
                    change.lo  = -10,
                    change.hi  = 10
                        )
   
cap.mcd <- AvgPreds(mod        = mod.mcd,
                    varcov     = vcov.mcd,
                    var.change = 13,
                    pol.rel    = TRUE,
                    change.vec = range.public
                    )

##########################################
# Average effects for industrialization, #
# capitalism, and democracy in the       #
# Gartzke model                          #
##########################################

ind.open <- AvgPreds(mod        = mod.open,
                     varcov     = vcov.open,
                     var.change = 2,
                     pol.rel    = TRUE,
                     change.vec = range.ind
                     )
 
dem.open <- AvgPreds(mod        = mod.open,
                     varcov     = vcov.open,
                     var.change = 7,
                     pol.rel    = TRUE,
                     change.lo  = -10,
                     change.hi  = 10
                     )

cap.open <- AvgPreds(mod        = mod.open,
                     varcov     = vcov.open,
                     var.change = 13,
                     pol.rel    = TRUE,
                     change.vec = range.open
                     )

##########################################
# Average effects for industrialization, #
# capitalism, and democracy in the       #
# Mousseau model                         #
##########################################

ind.cont <- AvgPreds(mod        = mod.cont,
                     varcov     = vcov.cont,
                     var.change = 2,
                     pol.rel    = TRUE,
                     change.vec = range.ind
                     )
 
dem.cont <- AvgPreds(mod        = mod.cont,
                     varcov     = vcov.cont,
                     var.change = 7,
                     pol.rel    = TRUE,
                     change.lo  = -10,
                     change.hi  = 10
                     )

cap.cont <- AvgPreds(mod        = mod.cont,
                     varcov     = vcov.cont,
                     var.change = 13,
                     pol.rel    = TRUE,
                     change.vec = range.cie
                    )

##########################
# Plot results from each #
##########################

res.df <- data.frame(pred   = c(ind.base$pred,
                                dem.base$pred,
                                ind.mcd$pred,
                                dem.mcd$pred,
                                cap.mcd$pred,
                                ind.open$pred,
                                dem.open$pred,
                                cap.open$pred,
                                ind.cont$pred,
                                dem.cont$pred,
                                cap.cont$pred
                                ),
                     lo      = c(ind.base$lo,
                                 dem.base$lo,
                                 ind.mcd$lo,
                                 dem.mcd$lo,
                                 cap.mcd$lo,
                                 ind.open$lo,
                                 dem.open$lo,
                                 cap.open$lo,
                                 ind.cont$lo,
                                 dem.cont$lo,
                                 cap.cont$lo
                                 ),
                     hi     = c(ind.base$hi,
                                dem.base$hi,
                                ind.mcd$hi,
                                dem.mcd$hi,
                                cap.mcd$hi,
                                ind.open$hi,
                                dem.open$hi,
                                cap.open$hi,
                                ind.cont$hi,
                                dem.cont$hi,
                                cap.cont$hi
                                ),
                     Variable = c(rep("Industrialization",
                                  times = length(ind.base$pred)
                                  ),
                                rep("Democracy",
                                    times = length(dem.base$pred)
                                    ),
                                rep("Industrialization",
                                    times = length(ind.mcd$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.mcd$pred)
                                    ),
                                rep("Capitalism",
                                    times = length(cap.mcd$pred)
                                    ),
                                rep("Industrialization",
                                    times = length(ind.open$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.open$pred)
                                    ),
                                rep("Capitalism",
                                    times = length(cap.open$pred)
                                    ),
                                rep("Industrialization",
                                    times = length(ind.cont$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.cont$pred)
                                    ),
                                rep("Capitalism",
                                    times = length(ind.cont$pred)
                                    )
                                ),
                     change = c(UnitLength(length(ind.base$pred)),
                                UnitLength(length(dem.base$pred)),
                                UnitLength(length(ind.mcd$pred)),
                                UnitLength(length(dem.mcd$pred)),
                                UnitLength(length(cap.mcd$pred)),
                                UnitLength(length(ind.open$pred)),
                                UnitLength(length(dem.open$pred)),
                                UnitLength(length(cap.open$pred)),
                                UnitLength(length(ind.cont$pred)),
                                UnitLength(length(dem.cont$pred)),
                                UnitLength(length(cap.cont$pred))
                                ),
                     model  = c(rep("Baseline",
                                    times = length(ind.base$pred) + 
                                            length(dem.base$pred)
                                    ),
                                rep("Public\nProperty",
                                    times = length(ind.mcd$pred) +
                                            length(dem.mcd$pred) +
                                            length(cap.mcd$pred)
                                    ),
                                rep("Capital\nOpenness",
                                    times = length(ind.open$pred) +
                                            length(dem.open$pred) +
                                            length(cap.open$pred)
                                    ),
                                rep("Contract\nIntensiveness",
                                    times = length(ind.cont$pred) +
                                            length(dem.cont$pred) +
                                            length(cap.cont$pred)
                                    )
                                )
                     )

res.df$model <- factor(res.df$model, 
                       levels = unique(as.character(res.df$model))
                       )
res.df$Variable <- factor(res.df$Variable,
                          levels = unique(as.character(res.df$Variable))
                          )

############
# Figure 2 #
############

pdf("dyadic_preds.pdf",
    width  = 9,
    height = 7
    )
  ggplot(data = res.df,
         aes(x        = change,
             y        = pred,
             colour   = Variable,
             linetype = Variable
             )
         ) +
    labs(x = "Shift Across Variable Range",
         y = "Probability of Fatal Conflict"
         ) +
    geom_line(size = 1.25,
              ) +
    geom_ribbon(data = res.df,
                aes(ymin = lo,
                    ymax = hi,
                    fill = Variable
                    ),
                alpha = 0.3,
                colour = NA
    ) +
    scale_x_continuous(breaks = NULL) +
    facet_wrap( ~ model,
                  ncol   = 2,
                  scales = "free"
               ) +  
    theme_bw() 
dev.off()
