###############################################
# Code to replicate monadic analysis for:     #
# "A Modern Peace? Schumpeter, the Decline of #    
# Conﬂict, and the Investment-War Tradeoff"   #
#                                             #
# J. Tyson Chatagnier and Emanuele Castelli   #
#                                             #
# Forthcoming in Political Research Quarterly #
#                                             #
# File created 29 July 2016                   #
###############################################

rm(list = ls())

library(foreign)
library(ggplot2)
library(apsrtable)

# setwd("path")
                        # Use to change default directory

source("modern_peace_auxiliary_functions.R")

########
# Data #
########

monadic <- read.dta("monadic_data_set.dta")
                        # Read in the relevant data.

################################
# Add and manipulate variables #
################################

mod.df <- data.frame(industry    = monadic$industry,
                     service     = monadic$service,
                     gdp         = log(monadic$gdp),
                        # Our GDPpc measure is unlogged, so log it here
                     M           = monadic$m * 10,
                        # Rescale this variable purely for aesthetics
                     majpow      = monadic$majpow * 10,
                     polity      = monadic$polity2,
                     trade       = log(monadic$trade),
                        # Our trade measure is unlogged, so log it here
                     nuclear     = monadic$nuclear * 10,
                     t           = monadic$t_fatal,
                     t2          = (monadic$t_fatal ^ 2),
                     t3          = (monadic$t_fatal ^ 3),
                     public      = monadic$public,
                     open        = monadic$open,
                     contract    = monadic$cie,
                     mid         = monadic$fatal_onset,
                     ccode       = monadic$ccode,
                     year        = monadic$year
                     )
                        # Create a data frame that has the relevant variables,
                        # including some useful manipulations.

mod.df$trade <- ifelse(is.na(mod.df$trade), 0, mod.df$trade)
                        # Turn NAs in the trade data into zeros.
                        # Results should be robust to MI.

###################
# Estimate models #
###################

mod.base <- glm(mid ~ industry + service + gdp + M + 
                      majpow + polity + trade + nuclear + 
                      t + t2 + t3,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

mod.mcd <- glm(mid ~ industry + service + gdp + M + 
                     majpow + polity + trade + nuclear + 
                     public + t + t2 + t3,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

mod.open <- glm(mid ~ industry + service + gdp + M + 
                      majpow + polity + trade + nuclear +
                      open + t + t2 + t3,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

mod.cont <- glm(mid ~ industry + service + gdp + M + 
                      majpow + polity + trade + nuclear + 
                      contract + t + t2 + t3,
                family = binomial(link = "logit"),
                data   = mod.df,
                x      = TRUE
                )

vcov.base <- ClusterVarCov(mod.base,
                           .data   = mod.df,
                           cluster = mod.df$ccode
                           )

vcov.mcd <- ClusterVarCov(mod.mcd,
                          .data   = mod.df,
                          cluster = mod.df$ccode
                          )

vcov.open <- ClusterVarCov(mod.open,
                           .data   = mod.df,
                           cluster = mod.df$ccode
                           )

vcov.cont <- ClusterVarCov(mod.cont,
                           .data   = mod.df,
                           cluster = mod.df$ccode
                           )

###########
# Table 1 #
###########

mod.base$se <- sqrt(diag(vcov.base))
mod.mcd$se <- sqrt(diag(vcov.mcd))
mod.open$se <- sqrt(diag(vcov.open))
mod.cont$se <- sqrt(diag(vcov.cont))


apsrtable(mod.base,
          mod.mcd,
          mod.open,
          mod.cont,
          stars = "default",
          se = "robust"
          )
                        # Note: formatting is not identical to the
                        # table in the article, but the substance
                        # is the same.

#####################################
# Save models to generate plots for #
# appendix later.                   #
#####################################

monad.base <- mod.base
monad.mcd <- mod.mcd
monad.open <- mod.open
monad.cont <- mod.cont

vcov.base.mon <- vcov.base
vcov.mcd.mon <- vcov.mcd
vcov.open.mon <- vcov.open
vcov.cont.mon <- vcov.cont
                        # Rename to distinguish from dyadic models

save(monad.base,
     monad.mcd,
     monad.open,
     monad.cont,
     vcov.base.mon,
     vcov.mcd.mon,
     vcov.open.mon,
     vcov.cont.mon,
     file = "monadic_mods.rdata"
     )
                        # Save these data frames

#########################################
# Average effects for industrialization #
# and democracy in the baseline model   #
#########################################

ind.base <- AvgPreds(mod        = mod.base,
                     varcov     = vcov.base,
                     var.change = 2,
                     change.vec = MinMax(mod.df$industry)
                     )

dem.base <- AvgPreds(mod        = mod.base,
                     varcov     = vcov.base,
                     var.change = 7,
                     change.lo  = -10,
                     change.hi  = 10
                     )
 
##########################################
# Average effects for industrialization, #
# capitalism, and democracy in the       #
# McDonald model                         #
##########################################


ind.mcd <- AvgPreds(mod        = mod.mcd,
                    varcov     = vcov.mcd,
                    var.change = 2,
                    change.vec = MinMax(mod.df$industry)
                    )

dem.mcd <- AvgPreds(mod        = mod.mcd,
                    varcov     = vcov.mcd,
                    var.change = 7,
                    change.lo  = -10,
                    change.hi  = 10
                    )
   
cap.mcd <- AvgPreds(mod        = mod.mcd,
                    varcov     = vcov.mcd,
                    var.change = 10,
                    change.vec = MinMax(mod.df$public)
                    )

##########################################
# Average effects for industrialization, #
# capitalism, and democracy in the       #
# Gartzke model                          #
##########################################

ind.open <- AvgPreds(mod        = mod.open,
                     varcov     = vcov.open,
                     var.change = 2,
                     change.vec = MinMax(mod.df$industry)
                     )
 
dem.open <- AvgPreds(mod        = mod.open,
                     varcov     = vcov.open,
                     var.change = 7,
                     change.lo  = -10,
                     change.hi  = 10
                     )

cap.open <- AvgPreds(mod        = mod.open,
                     varcov     = vcov.open,
                     var.change = 10,
                     change.vec = MinMax(mod.df$open)
                     )

##########################################
# Average effects for industrialization, #
# capitalism, and democracy in the       #
# Mousseau model                         #
##########################################

ind.cont <- AvgPreds(mod        = mod.cont,
                     varcov     = vcov.cont,
                     var.change = 2,
                     change.vec = MinMax(mod.df$industry)
                     )
 
dem.cont <- AvgPreds(mod        = mod.cont,
                     varcov     = vcov.cont,
                     var.change = 7,
                     change.lo  = -10,
                     change.hi  = 10
                     )

cap.cont <- AvgPreds(mod        = mod.cont,
                     varcov     = vcov.cont,
                     var.change = 10,
                     change.vec = MinMax(mod.df$contract)
                    )

##########################
# Plot results from each #
##########################

res.df <- data.frame(pred   = c(ind.base$pred,
                                dem.base$pred,
                                ind.mcd$pred,
                                dem.mcd$pred,
                                cap.mcd$pred,
                                ind.open$pred,
                                dem.open$pred,
                                cap.open$pred,
                                ind.cont$pred,
                                dem.cont$pred,
                                cap.cont$pred
                                ),
                     lo      = c(ind.base$lo,
                                dem.base$lo,
                                ind.mcd$lo,
                                dem.mcd$lo,
                                cap.mcd$lo,
                                ind.open$lo,
                                dem.open$lo,
                                cap.open$lo,
                                ind.cont$lo,
                                dem.cont$lo,
                                cap.cont$lo
                                ),
                     hi     = c(ind.base$hi,
                                dem.base$hi,
                                ind.mcd$hi,
                                dem.mcd$hi,
                                cap.mcd$hi,
                                ind.open$hi,
                                dem.open$hi,
                                cap.open$hi,
                                ind.cont$hi,
                                dem.cont$hi,
                                cap.cont$hi
                                ),
                     Variable = c(rep("Industrialization",
                                    times = length(ind.base$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.base$pred)
                                    ),
                                rep("Industrialization",
                                    times = length(ind.mcd$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.mcd$pred)
                                    ),
                                rep("Capitalism",
                                    times = length(cap.mcd$pred)
                                    ),
                                rep("Industrialization",
                                    times = length(ind.open$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.open$pred)
                                    ),
                                rep("Capitalism",
                                    times = length(cap.open$pred)
                                    ),
                                rep("Industrialization",
                                    times = length(ind.cont$pred)
                                    ),
                                rep("Democracy",
                                    times = length(dem.cont$pred)
                                    ),
                                rep("Capitalism",
                                    times = length(ind.cont$pred)
                                    )
                                ),
                     change = c(UnitLength(length(ind.base$pred)),
                                UnitLength(length(dem.base$pred)),
                                UnitLength(length(ind.mcd$pred)),
                                UnitLength(length(dem.mcd$pred)),
                                UnitLength(length(cap.mcd$pred)),
                                UnitLength(length(ind.open$pred)),
                                UnitLength(length(dem.open$pred)),
                                UnitLength(length(cap.open$pred)),
                                UnitLength(length(ind.cont$pred)),
                                UnitLength(length(dem.cont$pred)),
                                UnitLength(length(cap.cont$pred))
                                ),
                     model  = c(rep("Baseline",
                                    times = length(ind.base$pred) + 
                                            length(dem.base$pred)
                                    ),
                                rep("Public\nProperty",
                                    times = length(ind.mcd$pred) +
                                            length(dem.mcd$pred) +
                                            length(cap.mcd$pred)
                                    ),
                                rep("Capital\nOpenness",
                                    times = length(ind.open$pred) +
                                            length(dem.open$pred) +
                                            length(cap.open$pred)
                                    ),
                                rep("Contract\nIntensiveness",
                                    times = length(ind.cont$pred) +
                                            length(dem.cont$pred) +
                                            length(cap.cont$pred)
                                    )
                                )
                     )

res.df$model <- factor(res.df$model, 
                       levels = unique(as.character(res.df$model))
                       )
res.df$Variable <- factor(res.df$Variable,
                          levels = unique(as.character(res.df$Variable))
                          )

############
# Figure 1 #
############

pdf("monadic_preds.pdf",
    width  = 9,
    height = 7
    )
  ggplot(data = res.df,
         aes(x        = change,
             y        = pred,
             colour   = Variable,
             linetype = Variable
             )
         ) +
    labs(x = "Shift Across Variable Range",
         y = "Probability of Fatal Conflict"
         ) +
    geom_line(size = 1.25,
              ) +
    geom_ribbon(data = res.df,
                aes(ymin = lo,
                    ymax = hi,
                    fill = Variable
                    ),
                alpha = 0.3,
                colour = NA
    ) +
    scale_x_continuous(breaks = NULL) +
    facet_wrap( ~ model,
                  ncol   = 2,
                  scales = "free"
               ) +  
    theme_bw() 
dev.off()

